# Generated by roxygen2: do not edit by hand

S3method("[",hmatrix)
S3method(coef,FDboost)
S3method(cvrisk,FDboost)
S3method(cvrisk,FDboostLSS)
S3method(factorize,FDboost)
S3method(fitted,FDboost)
S3method(getArgvals,hmatrix)
S3method(getArgvalsLab,hmatrix)
S3method(getId,hmatrix)
S3method(getIdLab,hmatrix)
S3method(getTime,hmatrix)
S3method(getTimeLab,hmatrix)
S3method(getX,hmatrix)
S3method(getXLab,hmatrix)
S3method(mstop,validateFDboost)
S3method(plot,FDboost)
S3method(plot,bootstrapCI)
S3method(plot,validateFDboost)
S3method(predict,FDboost)
S3method(predict,FDboost_fac)
S3method(print,FDboost)
S3method(print,bootstrapCI)
S3method(print,validateFDboost)
S3method(residuals,FDboost)
S3method(stabsel,FDboost)
S3method(summary,FDboost)
S3method(update,FDboost)
export("%A%")
export("%A0%")
export("%Xa0%")
export("%Xc%")
export(FDboost)
export(FDboostLSS)
export(applyFolds)
export(bbsc)
export(bconcurrent)
export(bfpc)
export(bhist)
export(bhistx)
export(bolsc)
export(bootstrapCI)
export(brandomc)
export(bsignal)
export(clr)
export(cvLong)
export(cvMa)
export(factorize)
export(funMRD)
export(funMSE)
export(funRsquared)
export(funplot)
export(getArgvals)
export(getArgvalsLab)
export(getId)
export(getIdLab)
export(getTime)
export(getTimeLab)
export(getX)
export(getXLab)
export(hmatrix)
export(integrationWeights)
export(integrationWeightsLeft)
export(is.hmatrix)
export(o_control)
export(plotPredCoef)
export(plotPredicted)
export(plotResiduals)
export(reweightData)
export(subset_hmatrix)
export(truncateTime)
export(validateFDboost)
export(wide2long)
exportClasses(FDboost_fac)
import(Matrix)
import(mboost)
import(methods)
importFrom(MASS,Null)
importFrom(MASS,ginv)
importFrom(Matrix,rankMatrix)
importFrom(gamboostLSS,GaussianLSS)
importFrom(gamboostLSS,GaussianMu)
importFrom(gamboostLSS,GaussianSigma)
importFrom(gamboostLSS,cvrisk.mboostLSS)
importFrom(gamboostLSS,make.grid)
importFrom(gamboostLSS,mboostLSS_fit)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,contour)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,persp)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(methods,setOldClass)
importFrom(mgcv,gam)
importFrom(mgcv,s)
importFrom(parallel,mclapply)
importFrom(splines,bs)
importFrom(splines,splineDesign)
importFrom(stabs,stabsel)
importFrom(stabs,stabsel_parameters)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,model.weights)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms.formula)
importFrom(stats,variable.names)
importFrom(utils,getS3method)
importFrom(utils,packageDescription)
importFrom(utils,relist)
importFrom(zoo,na.locf)
