
Code for simulation study presented in 
Brockhaus, S., Scheipl, F., Hothorn, T. and Greven, S. (2015). The functional linear array model. Statistical Modelling, 15(3), 279-300.

To run the simulation use run_FDboost.R and run_pffr.R (call helper functions in simfuns.R)
Plots of the simulated data are generated by run_PlotModels.R 
Plots and tables of the results are computed by analyze_results.R 

In run-time.R a small simulation study is conducted to compare the run-time of the array model against the non-array model. 

The parallelization only works on Linux, not on Windows. 